# -*- coding: utf-8 -*-
from django.contrib import admin

from .models import *
from transmeta import canonical_fieldname



class ArticleAdmin(admin.ModelAdmin):
    def formfield_for_dbfield(self, db_field, **kwargs):
        field = super(ArticleAdmin, self).formfield_for_dbfield(db_field, **kwargs)
        db_fieldname = canonical_fieldname(db_field)
        return field


    fieldsets = [
        # ('Overview', {'fields': ['title', 'slug', 'status', 'desc', 'pubDate']}),
        # ('Author', {'fields': ['authorProfileModel', 'authorProfileId']}),
        # ('Content', {'fields': ['content', 'title_image']}),
        # ('Meta', {'fields': ['seoKeyWords_de', 'seoDesc', 'tags']}),
        # ('Options', {'fields': ['comments_enabled']})
    ]

    date_hierarchy = 'pubDate'
    
    def number_of_words(self, obj):
        return u'%s' %  obj.number_of_words()
    #number_of_words.admin_order_field = 'number_of_words'
        
    # prepopulated_fields = {'slug':('title',),}

    list_display = ('title', 'number_of_words', 'status', 'pubDate', 'addedDate', 'updatedDate')
    readonly_fields = ('addedDate', 'updatedDate',)
    list_filter = ('status', 'authorProfileId', )
    search_fields = ['title', 'slug', 'desc']
    
    if "redactormedia" in settings.INSTALLED_APPS:
        from redactormedia.widgets import RedactorWithMediaEditor, AdminRedactorWithMediaEditor
        from django.db import models
    
        formfield_overrides = {
            models.TextField: {'widget': AdminRedactorWithMediaEditor},
        }
        
admin.site.register(Article, ArticleAdmin)
admin.site.register(Tag)
admin.site.register(Comment)
admin.site.register(AuthorProfile)



# -*- coding: utf-8 -*-
import datetime
import urllib

from django.shortcuts import render_to_response
from django.template import RequestContext
from django.contrib.contenttypes.models import ContentType
from django.shortcuts import get_object_or_404
from django.utils import timezone

from .models import *

from .helperFunctions import getLatestArticles
from .helperFunctions import getArticlesByDate 
from .helperFunctions import getYearCount
from .helperFunctions import getLatestArticlesByAuthor




def listArticles(request):
    """
    """
    articles = getLatestArticles(100)
    return render_to_response('copywriting/copywritingIndex.html', {'articles': articles,
                                                                    'yearCount': getYearCount(),
                                                                   }, context_instance=RequestContext(request))

def listArticlesByAuthor(request, author):
    """
    author is the username of the user in the author model.
    """
    authorProfile = AuthorProfile.objects.get(user__username=author)
    articles = getLatestArticlesByAuthor(ContentType.objects.get_for_model(authorProfile), authorProfile.id, 100)
    return render_to_response('copywriting/copywritingIndex.html', {'articles': articles,
                                                                    'yearCount': getYearCount(),
                                                                    'authorProfile': authorProfile,
                                                                   }, context_instance=RequestContext(request))

    
def listArticlesByYear(request, requestYear):
    """
    """
    articles = getArticlesByDate(year=requestYear)
    return render_to_response('copywriting/copywritingIndex.html', {'articles': articles,
                                                                    'yearCount': getYearCount(),
                                                                   }, context_instance=RequestContext(request))


def listArticlesByYearMonth(request, requestYear, requestMonth):
    """
    """
    articles = getArticlesByDate(year=requestYear, month=requestMonth)
    return render_to_response('copywriting/copywritingIndex.html', {'articles': articles,
                                                                    'yearCount': getYearCount(),
                                                                   }, context_instance=RequestContext(request))


def listArticlesByYearMonthDay(request, requestYear, requestMonth, requestDay):
    """
    """
    articles = getArticlesByDate(year=requestYear, month=requestMonth, day=requestDay)
    return render_to_response('copywriting/copywritingIndex.html', {'articles': articles,
                                                                    'yearCount': getYearCount(),
                                                                   }, context_instance=RequestContext(request))


def withTag(request, in_tag):
    lTag = urllib.unquote(in_tag)
    tags = Tag.objects.filter(name=lTag)
    articles = Article.objects.filter(tags__in=tags, status=Article.PUBLISHED, pubDate__lte=timezone.now()).order_by('-pubDate')

    return render_to_response("copywriting/copywritingIndex.html", {'tag': in_tag,
                                                                    'articles': articles,
                                                                    'yearCount': getYearCount(),
                                                                   }, context_instance=RequestContext(request))


def showArticle(request, slug):
    """
    """
    if request.user.is_staff or request.user.is_superuser:
        article = get_object_or_404(Article, slug=slug)
    else:
        article = get_object_or_404(Article, slug=slug, status=Article.PUBLISHED)

    if article:
        latestArticlesList = getLatestArticlesByAuthor(article.authorProfileModel, article.authorProfileId, 5, slug)

    return render_to_response('copywriting/copywritingArticle.html', {'article': article,
                                                                      'latestArticlesList': latestArticlesList,
                                                                     }, context_instance=RequestContext(request))



# -*- coding: utf-8 -*-
from datetime import datetime
from .models import Article, Tag
from django.utils import timezone


def getLatestArticlesByTag(amount=5, tagString=None):
    """
    Returns the latest n (=amount) published Articles that have all tags.
    Tags is tagString exploded using ",".
    
    """
    articles = getArticles()
    tags = []
    
    if articles:
        tags = tagString.split(",")
    
    for tag in tags:
        articles = articles.filter(tags__name=tag)
 
    return articles[:amount]
    

def getArticles():
    """
    Returns all published Articles
    """
    try:
        articles = Article.objects.filter(status=Article.PUBLISHED, pubDate__lte=timezone.now()).order_by('-pubDate')
    except:
        articles = None

    return articles


def getLatestArticles(amount=5):
    """
    Returns the latest n (=amount) published Articles
    """
    articles = getArticles()
    
    if amount == 1 and articles:
        return articles[0]
    
    return articles[:amount]


def getArticlesByAuthor(authorModel, authorId, toExclude=None):
    """
    Returns all published Articles for an author
    """
    articles = getArticles().filter(authorProfileModel=authorModel, authorProfileId=authorId).exclude(slug=toExclude)
    
    return articles


def getLatestArticlesByAuthor(authorModel, authorId, amount=5, toExclude=None):
    """
    Returns the latest n (=amount) published Articles for an author
    """
    articles = getArticlesByAuthor(authorModel, authorId, toExclude=toExclude)
    
    return articles[:amount]


def getTags():
    """
    Returns all Tags that are used, Tags not linked to published Articles are excluded
    """
    tags_ids = []

    articles = getArticles()
    
    for article in articles:
        for tag in article.tags.all():
            tags_ids.append(tag.id)

    return Tag.objects.filter(id__in=tags_ids).order_by('name')


def getTagsByAuthor(authorModel, authorId):
    """
    Returns all Tags that an author has used in this articles
    """
    tags_ids = []

    author_articles = getArticlesByAuthor(authorModel, authorId)
    
    for article in author_articles:
        for tag in article.tags.all():
            tags_ids.append(tag.id)

    return Tag.objects.filter(id__in=tags_ids).order_by('name')


def getYearCount():
    """
    Return a list of years an count of Articles for this year like:
    yearCount = [[2012, 3],[2011, 43],[2010, 74]]"""
    try:    
        years = Article.objects.datetimes('pubDate', 'year')
    except AttributeError:
        years = Article.objects.dates('pubDate', 'year')
    
    yearCount = []

    for year in years:
        year = int(year.year)
        count = getArticles().filter(pubDate__year=year).count()
        yearCount.append([year, count])

    return sorted(yearCount, reverse=True)


def getArticlesByYear(year):
    try:
        articles = getArticles().filter(pubDate__year=year)
    except Exception:
        articles = None
    return articles


def getArticlesByYearMonth(year, month):
    try:
        articles = getArticles().filter(pubDate__year=year, pubDate__month=month)
    except Exception:
        articles = None
    return articles


def getArticlesByYearMonthDay(year, month, day):
    try:
        articles = getArticles().filter(pubDate__year=year, pubDate__month=month, pubDate__day=day).order_by('-pubDate')
    except Exception:
        articles = None
    return articles


def getArticlesByDate(**kwargs):
    """based on the keywords retuns a list of Articles for a specific date
    keywords can be: year, month and day"""
    if 'year' in kwargs and 'month' in kwargs and 'day' in kwargs:
        articles = getArticlesByYearMonthDay(kwargs['year'], kwargs['month'], kwargs['day'])
    elif 'year' in kwargs and 'month' in kwargs:
        articles = getArticlesByYearMonth(kwargs['year'], kwargs['month'])
    elif 'year' in kwargs:
        articles = getArticlesByYear(kwargs['year'])
    else:
        articles = None

    return articles



# -*- coding: utf-8 -*-
import datetime
import traceback 
import sys

from django.db import models
from django.contrib.sitemaps import ping_google
from django.conf import settings
from django.utils.translation import ugettext as _
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from compat import GenericForeignKey
from django.utils.html import strip_tags 
from django.core.urlresolvers import reverse
# from django.contrib.comments.models import Comment

from transmeta import TransMeta
# from tagging.models import Tag

from arimagebucket.models import *
from django.dispatch import receiver
from .signals import ready_to_publish, ready_to_review


from django.utils import timezone
 
class Tag(models.Model):
    name = models.CharField(max_length=200)

    def __unicode__(self):
        return self.name


class Article(models.Model):
    """
    """
    __metaclass__ = TransMeta
    
    PUBLISHED = 1 # Compatibility mode
    READY_TO_PUBLISH = 2
    READY_TO_REVIEW = 3
    DRAFT = 0
    
    STATE_OPTIONS = ((DRAFT, _('Draft')),
                     (READY_TO_REVIEW, _('Ready To Review')),
                     (READY_TO_PUBLISH, _('Ready To Publish')),
                     (PUBLISHED, _('Published')),
                    )
                    
    # Overview
    title = models.CharField(verbose_name="Title", max_length=250, help_text="Short title (<250 chars)", default="")
    status = models.IntegerField('Visibility', default=0, choices=STATE_OPTIONS)
    slug = models.SlugField(max_length=80, unique=True, help_text="Eg.: 'swiss-drink-promo' (link to news page whould be '/articles/cool-new-feature/' if fullContent !='')", default="")
    desc = models.CharField('Description', max_length=250, help_text="Short desc (<250 chars)", default="")
    authorProfileModel = models.ForeignKey(ContentType)
    authorProfileId = models.PositiveIntegerField()
    author = GenericForeignKey('authorProfileModel', 'authorProfileId')
    pubDate = models.DateTimeField('Date To publish the Article', default=timezone.now, help_text="specify article date")
    addedDate = models.DateTimeField('Date created', auto_now_add=True)
    updatedDate = models.DateTimeField('Date last Modified', auto_now=True)

    # Content
    title_image = models.ForeignKey(ImageBucketObject, help_text="Image for Facebook & Co.",  null=True, blank=True)
    content = models.TextField(verbose_name="Content", blank=True, null=True, help_text="Content with double rendering.", default="")
    
    # Meta
    seoKeyWords = models.CharField(verbose_name='Keywords for meta tags', max_length=250, help_text="SEO: Keywords, separated with ', '", default="arteria")
    seoDesc = models.CharField(verbose_name='Desc for meta tags', max_length=250, help_text="SEO: Description, leave empty for post's description", default="arteria")
    tags = models.ManyToManyField(Tag, blank=True)
    # tags = TagField(help_text="Tags, getrennt durch 'spaces' (aktuell), siehe auch https://arteria.jira.com/wiki/display/ART/Blog+Tags")

    # Comments
    comments_enabled = models.BooleanField(default=True, help_text="Activate Comments")

    class Meta:
        translate = ('title', 'desc', 'content', 'seoKeyWords', 'seoDesc', )

    # def set_tags(self, tags):
    #     Tag.objects.update_tags(self, tags)

    # def get_tags(self):
    #     return Tag.objects.get_for_object(self)

    
    def number_of_words(self):
        ''' 
        '''
        plain = strip_tags(self.content) 
        return len(plain.split(' '))
        
        
    def __unicode__(self):
        """
        """
        return "%s (%i words)" % (self.slug, self.number_of_words())


    def get_absolute_url(self):
        """
        """
        return reverse('copywriting.views.showArticle', None, [str(self.slug)])


    def save(self, force_insert=False, force_update=False):
        isFirstSave = False
        if self.pk is not None:
            orig = Article.objects.get(pk=self.pk)
            if orig.status != self.status:
                if self.status == self.READY_TO_REVIEW:
                    ready_to_review.send(sender=None, articleID = self.id)
                if self.status == self.READY_TO_PUBLISH:
                    ready_to_publish.send(sender=None, articleID = self.id)
                     
        else:
            isFirstSave = True
                 
        super(Article, self).save(force_insert, force_update)
        if isFirstSave is True:
            if self.status == self.READY_TO_REVIEW:
                ready_to_review.send(sender=None, articleID = self.id)
            if self.status == self.READY_TO_PUBLISH:
                ready_to_publish.send(sender=None, articleID = self.id)  
        try:
            if getattr(settings, 'DEBUG', False) is False:
                ping_google()
        except Exception:
            pass
    

class AuthorProfile(models.Model):
    user = models.OneToOneField(User)
    bio = models.TextField(help_text="Steckbrief", 
        null=True, 
        blank=True)
    shortBio = models.TextField(help_text="Steckbrief (Teaser)",
        null=True, 
        blank=True)
    twitter = models.CharField(max_length=100, 
        null=True, 
        blank=True,
        help_text="twitter benutzername ohne '@' - z.B. arteria_ch")
    linkedin = models.CharField(max_length=100, null=True, blank=True,
        help_text="link zum linkedin Profil - z.B. http://www.linkedin.com/profile/view?id=209942560")
    facebook = models.CharField(max_length=100,
        null=True,
        blank=True,
        help_text="link zum facebook Profil - z.B. https://www.facebook.com/arteria.ch")
    xing = models.CharField(max_length=100, null=True, blank=True,
        help_text="link zum xing Profil - z.B. https://www.xing.com/profile/Walter_Renner5")
    google_plus = models.CharField(max_length=100, null=True, blank=True,
        help_text="link to googple Plus Profile")
    sex = models.IntegerField('Sex', choices=((0, "neutral"), (1, "weiblich"), (2, "maennlich"), ), default=0)

    def __unicode__(self):
        return self.user.username

    def get_email(self):
        return self.user.email

    def get_full_name(self):
        return "%s %s" %(self.user.first_name, self.user.last_name)

    def get_google_plus_url(self):
        return self.google_plus


class Comment(models.Model):
    author = models.CharField(max_length=100)
    email = models.CharField(max_length=200)
    content = models.TextField()
    created = models.DateTimeField(auto_now_add=True)
    article = models.ForeignKey(Article)

    def __unicode__(self):
        return self.article.title

# class Comment(Comment):
#     article = models.ForeignKey(Article, blank=False, )
#     content = models.CharField(max_length=300)

#     def get_comment_model(self):
#         # Use our custom comment model instead of the built-in one.
#         return Comment

if 'watson' in settings.INSTALLED_APPS:
    import watson
    watson.register(Article)



# -*- coding: utf-8 -*-
from django import template
from copywriting.helperFunctions import getLatestArticles
from copywriting.helperFunctions import getLatestArticlesByTag
from copywriting.helperFunctions import getTags
from copywriting.helperFunctions import getArticles

register = template.Library()


@register.filter
def get_latest_slug(request):
    latest = getLatestArticles(1)
    return latest.slug


@register.assignment_tag
def get_latest_articles_by_tag( *args, **kwargs):
    """
    Usage:
    
    {% get_latest_articles_by_tag tagString="Technology,Biology,Match" amount=5 as articles %}
    
    {% for article in articles %}
        {{ article.title }}
    {% endfor %}
    
    Notes: If tagString is not defined or an empty string, 'getLatestArticles' will be called.
    """
    amount = kwargs.get('amount',5)
    tagString = kwargs.get('tagString', None)
    if tagString is None or tagString == '':
        return getLatestArticles(amount)
    return getLatestArticlesByTag(amount, tagString=tagString)
 

@register.filter
def get_latest_articles(request):
    return getLatestArticles(5)


@register.filter
def get_tags(request):
    return getTags()


@register.simple_tag
def get_gravatar_url_by_email(email, size=48):
    """
    Simply gets the Gravatar Url for the a email Adress. There is no rating or
    custom "not found" icon yet. Used with the Django comments.

    If no size is given, the default is 48 pixels by 48 pixels.

    Template Syntax::

        {% gravatarUrl comment.user_email [size] %}

    Example usage::

        {% gravatarUrl comment.user_email 48 %}
    """
    import urllib
    import hashlib

    url = "https://secure.gravatar.com/avatar.php?"
    url += urllib.urlencode({
        'gravatar_id': hashlib.md5(email).hexdigest(),
        'size': str(size)
    })

    return url



@register.assignment_tag
def next(current_Articel):
    articles = getArticles()
    newer_articles = []

    for article in articles:
        if article.pubDate > current_Articel.pubDate:
            newer_articles.append(article)

    if newer_articles == []:
        return False
        #return False

    else:
        next = newer_articles[0]

    for article in newer_articles:
        if article.pubDate < next.pubDate:
            next = article

    return next

@register.assignment_tag
def prev(current_Articel):
    articles = getArticles()
    older_articles = []

    for article in articles:
        if article.pubDate < current_Articel.pubDate:
            older_articles.append(article)

    if older_articles == []:
        return False
        #return False

    else:
        prev = older_articles[0]

    for article in older_articles:
        if article.pubDate > prev.pubDate:
            prev = article

    return prev

@register.assignment_tag
def getNewest(count=1):
    return getLatestArticles(count)



# -*- coding: utf-8 -*-
from django.contrib.syndication.views import Feed
from django.conf import settings
from .helperFunctions import getLatestArticles
import logging

logger = logging.getLogger(__name__)


class blogFeed(Feed):
    if hasattr(settings, 'FEED_SETTINGS'):
        title = settings.FEED_SETTINGS.get('title', "")
        link = settings.FEED_SETTINGS.get('link', "")
        description = settings.FEED_SETTINGS.get('description', "")
        author_email = settings.FEED_SETTINGS.get('author_email', "")
        author_name = settings.FEED_SETTINGS.get('author_name', "")
        author_link = settings.FEED_SETTINGS.get('author_link', "")
        feed_url = settings.FEED_SETTINGS.get('feed_url', "")
        categories = settings.FEED_SETTINGS.get('categories', "")
    else:
        logger.error('settings has no attribute FEED_SETTINGS that are required for the Feed')

    def items(self):
        return getLatestArticles(5)

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.content



from compat import patterns, url
from .feed import blogFeed

urlpatterns = patterns('copywriting',
    url(r'^feed\.rss$', blogFeed()),
    url(r'^feed/$', blogFeed()),
    url(r'^author/(?P<author>\w[^/]+)$', 'views.listArticlesByAuthor'),
    url(r'^author/(?P<author>\w[^/]+)/$', 'views.listArticlesByAuthor'),
    url(r'^tag/(?P<in_tag>\w[^/]+)$', 'views.withTag'),
    url(r'^tag/(?P<in_tag>\w[^/]+)/$', 'views.withTag'),
    # (r'^(?P<year>\d+)/(?P<month>\d+)/(?P<day>\d+)/$', 'views.listBlogEntriesByYearMonthDay'),
    url(r'^(?P<requestYear>\d+)/(?P<requestMonth>\d+)/$', 'views.listArticlesByYearMonth'),
    url(r'^(?P<requestYear>\d+)/$', 'views.listArticlesByYear'),
    url(r'^(?P<slug>[^\.]+)/$', 'views.showArticle'),
    url('^$', 'views.listArticles'),
)



# -*- coding: utf-8 -*-
from django.contrib.sitemaps import Sitemap
from .models import Article


class BlogSitemap(Sitemap):
    changefreq = "daily"
    priority = 0.5

    def items(self):
        return Article.objects.filter(status=Article.PUBLISHED).order_by('-pubDate')

    def lastmod(self, obj):
        return obj.pubDate
